import React from 'react';
import { mountWithContext } from '../../test-utilities';
import { VisuallyHidden } from '../VisuallyHidden';
import { Form } from './Form';
describe('<Form />', function () {
  it('defaults to an empty action, POST method, and no validation', function () {
    var form = mountWithContext( /*#__PURE__*/React.createElement(Form, {
      onSubmit: noop
    }));
    expect(form).toContainReactComponent('form', {
      action: '',
      method: 'POST',
      noValidate: true
    });
  });
  it('calls onSubmit when the form is submitted', function () {
    var onSubmit = jest.fn();
    var form = mountWithContext( /*#__PURE__*/React.createElement(Form, {
      onSubmit: onSubmit
    }));
    form.find('form').trigger('onSubmit', createSubmitEvent());
    expect(onSubmit).toHaveBeenCalled();
  });
  it('prevents default when the form is submitted', function () {
    var event = createSubmitEvent();
    var form = mountWithContext( /*#__PURE__*/React.createElement(Form, {
      onSubmit: noop
    }));
    form.find('form').trigger('onSubmit', event);
    expect(event.preventDefault).toHaveBeenCalled();
  });
  describe('implicitSubmit', function () {
    it('renders a visually hidden submit with the submit label by default', function () {
      var expectedLabel = 'Submit';
      var form = mountWithContext( /*#__PURE__*/React.createElement(Form, {
        onSubmit: noop
      }), {
        translate: function translate(key) {
          return key === 'submit' ? expectedLabel : 'abc';
        }
      });
      expect(form.find(VisuallyHidden)).toContainReactComponent('button', {
        type: 'submit',
        children: expectedLabel,
        tabIndex: -1,
        'aria-hidden': true
      });
    });
    it('renders a visually hidden submit with a custom label', function () {
      var expectedLabel = 'Complete purchase';
      var form = mountWithContext( /*#__PURE__*/React.createElement(Form, {
        onSubmit: noop,
        implicitSubmit: expectedLabel
      }));
      expect(form.find(VisuallyHidden)).toContainReactComponent('button', {
        type: 'submit',
        children: expectedLabel
      });
    });
    it('does not render a visually hidden submit when false', function () {
      var form = mountWithContext( /*#__PURE__*/React.createElement(Form, {
        onSubmit: noop,
        implicitSubmit: false
      }));
      expect(form).not.toContainReactComponent(VisuallyHidden);
      expect(form).not.toContainReactComponent('button', {
        type: 'submit'
      });
    });
    it('disables the submit when disabled is true', function () {
      var form = mountWithContext( /*#__PURE__*/React.createElement(Form, {
        onSubmit: noop,
        disabled: true
      }));
      expect(form.find(VisuallyHidden)).toContainReactComponent('button', {
        type: 'submit',
        disabled: true
      });
    });
  });
});

function createSubmitEvent() {
  return {
    preventDefault: jest.fn()
  };
}

function noop() {}