"use strict";

var _react = _interopRequireDefault(require("react"));

var _faker = _interopRequireDefault(require("faker"));

var _testUtilities = require("../../test-utilities");

var _Portal = require("../Portal");

var _Modal = require("./Modal");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('<Modal />', () => {
  const defaultIFrameSrc = _faker.default.internet.url();

  describe('open prop', () => {
    it('renders the modal when the open prop is true', async () => {
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        open: true,
        src: defaultIFrameSrc
      }));
      expect(modal).toContainReactComponent(_Portal.Portal);
    });
    it("doesn't render the modal when the open prop is undefined", async () => {
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        src: defaultIFrameSrc
      }));
      expect(modal).not.toContainReactComponent(_Portal.Portal);
    });
    it('calls onClose when open changes from true to false', async () => {
      const onCloseSpy = jest.fn();
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        onClose: onCloseSpy,
        src: defaultIFrameSrc,
        open: true
      }));
      modal.setProps({
        open: false
      });
      expect(onCloseSpy).toHaveBeenCalledTimes(1);
    });
  });
  it('renders an iframe is the src prop is passed', async () => {
    const iFrameSrc = _faker.default.internet.url();

    const iFrameName = _faker.default.random.words(2);

    const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
      src: iFrameSrc,
      iFrameName: iFrameName,
      open: true
    }));
    expect(modal).toContainReactComponent('iframe', {
      src: iFrameSrc,
      name: iFrameName
    });
  });
  it('renders its children if no src prop is passed', async () => {
    const content = _faker.default.random.words(2);

    const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
      open: true
    }, /*#__PURE__*/_react.default.createElement("p", null, content)));
    expect(modal).not.toContainReactComponent('iframe');
    expect(modal).toContainReactText(content);
  });
  it('renders a header if the title prop is passed', async () => {
    const title = _faker.default.random.words(2);

    const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
      src: defaultIFrameSrc,
      title: title,
      open: true
    }));
    expect(modal).toContainReactComponent('header');
    expect(modal).toContainReactText(title);
  });
  describe('when the escape key is pressed', () => {
    it('closes the modal', async () => {
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        src: defaultIFrameSrc,
        open: true
      }));
      const event = new KeyboardEvent('keydown', {
        key: 'Escape'
      });
      modal.act(() => document.dispatchEvent(event));
      expect(modal).not.toContainReactComponent(_Portal.Portal);
    });
    it('calls onClose callback', async () => {
      const onCloseSpy = jest.fn();
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        onClose: onCloseSpy,
        open: true
      }, "Hi"));
      const event = new KeyboardEvent('keydown', {
        key: 'Escape'
      });
      modal.act(() => document.dispatchEvent(event));
      expect(onCloseSpy).toHaveBeenCalledTimes(1);
    });
  });
  describe('when the close button is clicked', () => {
    it('closes the modal', async () => {
      var _modal$find;

      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        src: defaultIFrameSrc,
        title: "A modal",
        open: true
      }));
      (_modal$find = modal.find('button')) === null || _modal$find === void 0 ? void 0 : _modal$find.trigger('onClick');
      expect(modal).not.toContainReactComponent(_Portal.Portal);
    });
    it('calls onClose callback', async () => {
      var _modal$find2;

      const onCloseSpy = jest.fn();
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        title: "A modal",
        onClose: onCloseSpy,
        open: true
      }, "Hi"));
      (_modal$find2 = modal.find('button')) === null || _modal$find2 === void 0 ? void 0 : _modal$find2.trigger('onClick');
      expect(onCloseSpy).toHaveBeenCalledTimes(1);
    });
  });
  describe('blocking prop', () => {
    it("doesn't render the close button", async () => {
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        src: defaultIFrameSrc,
        title: "A modal",
        open: true,
        blocking: true
      }));
      expect(modal).not.toContainReactComponent('button');
    });
    it("doesn't close the modal when the escape key is pressed", async () => {
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        src: defaultIFrameSrc,
        open: true,
        blocking: true
      }));
      const event = new KeyboardEvent('keydown', {
        key: 'Escape'
      });
      modal.act(() => document.dispatchEvent(event));
      expect(modal).toContainReactComponent(_Portal.Portal);
    });
  });
  describe('long prop', () => {
    it("doesn't add the onLoad prop when present", async () => {
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        src: defaultIFrameSrc,
        open: true,
        long: true
      }));
      const iframe = modal.find('iframe');
      expect(iframe === null || iframe === void 0 ? void 0 : iframe.prop('onLoad')).toBeUndefined();
    });
    it('adds the onLoad prop when absent', async () => {
      const modal = await (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Modal.Modal, {
        src: defaultIFrameSrc,
        open: true
      }));
      const iframe = modal.find('iframe');
      expect(iframe === null || iframe === void 0 ? void 0 : iframe.prop('onLoad')).not.toBeUndefined();
    });
  });
});