"use strict";

var _react = _interopRequireDefault(require("react"));

var _faker = _interopRequireDefault(require("faker"));

var _testUtilities = require("../../test-utilities");

var _InlineError = require("../InlineError");

var _InlineStack = require("../InlineStack");

var _View = require("../View");

var _Select = require("./Select");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const defaultProps = {
  label: 'Country',
  options: [{
    value: 'CA',
    label: 'Canada'
  }, {
    value: 'US',
    label: 'United States'
  }, {
    value: 'UK',
    label: 'United Kingdom'
  }]
};
describe('<Select />', () => {
  it('renders a select and label', () => {
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, defaultProps));
    expect(select).toContainReactComponent('select');
    expect(select).toContainReactComponent('label');
  });
  it('wraps the component in an InlineStack when the labelPosition prop is inline', () => {
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      labelPosition: "inline"
    })));
    expect(select).toContainReactComponent(_InlineStack.InlineStack);
    expect(select).not.toContainReactComponent(_View.View);
  });
  it('wraps the component in a View when the labelPosition prop is not inline', () => {
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      labelPosition: "inside"
    })));
    expect(select).toContainReactComponent(_View.View);
    expect(select).not.toContainReactComponent(_InlineStack.InlineStack);
  });
  it('renders a select with a pre-selected value when the value prop is provided', () => {
    const value = 'UK';
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      value: value
    })));
    expect(select).toContainReactComponent('select', {
      value
    });
  });
  it('calls the onChange prop with the selected value when the selected value changes', () => {
    const onChangeSpy = jest.fn();
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      onChange: onChangeSpy
    })));
    select.find('select').trigger('onChange', {
      target: {
        value: 'UK'
      }
    });
    expect(onChangeSpy).toHaveBeenCalledWith('UK');
  });
  it('renders an error message when the error prop is provided', () => {
    const error = 'Select a country';

    const id = _faker.default.random.alphaNumeric();

    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      id: id,
      error: error
    })));
    expect(select).toContainReactComponent(_InlineError.InlineError, {
      controlID: id,
      children: error
    });
  });
  it('renders a disabled select when the disabled prop is true', () => {
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      disabled: true
    })));
    expect(select).toContainReactComponent('select', {
      disabled: true
    });
  });
  it('renders a disabled option when the placeholder is provided and selected', () => {
    const placeholder = 'Select a country';
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      placeholder: placeholder
    })));
    expect(select).toContainReactComponent('option', {
      disabled: true,
      value: _Select.PLACEHOLDER_VALUE,
      children: placeholder
    });
  });
  it('renders a disabled option when the placeholder is provided and not selected', () => {
    const placeholder = 'Select a country';
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      placeholder: placeholder,
      value: defaultProps.options[0].value
    })));
    expect(select).toContainReactComponent('option', {
      disabled: true,
      value: _Select.PLACEHOLDER_VALUE,
      children: placeholder
    });
  });
  it('renders a disabled option when the readonly prop is true', () => {
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      readonly: true
    })));
    expect(select).toContainReactComponent('option', {
      disabled: true
    });
  });
  it('renders a hidden and disabled option when the placeholder is provided, selected and similar to the label', () => {
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      placeholder: defaultProps.label
    })));
    expect(select).toContainReactComponent('option', {
      disabled: true,
      value: _Select.PLACEHOLDER_VALUE,
      hidden: true
    });
    expect(select).not.toContainReactComponent('option', {
      children: defaultProps.label
    });
  });
  it("doesn't render a hidden and disabled option when the placeholder is provided, not selected and similar to the label", () => {
    const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
      placeholder: defaultProps.label,
      value: defaultProps.options[0].value
    })));
    expect(select).not.toContainReactComponent('option', {
      disabled: true,
      value: _Select.PLACEHOLDER_VALUE,
      hidden: true
    });
  });
  describe('autocomplete', () => {
    it('sets the select autocomplete attribute to "off" when autocomplete is false', () => {
      const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
        autocomplete: false
      })));
      expect(select).toContainReactComponent('select', {
        autoComplete: 'none'
      });
    });
    it('sets the select autocomplete attribute to "on" when autocomplete is true', () => {
      const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
        autocomplete: true
      })));
      expect(select).toContainReactComponent('select', {
        autoComplete: 'on'
      });
    });
    it('sets the select autocomplete attribute to a string representation of an autocomplete descriptor', () => {
      const select = (0, _testUtilities.mountWithContext)( /*#__PURE__*/_react.default.createElement(_Select.Select, Object.assign({}, defaultProps, {
        autocomplete: {
          group: 'billing',
          field: 'country'
        }
      })));
      expect(select).toContainReactComponent('select', {
        autoComplete: 'billing country'
      });
    });
  });
});