"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const test_utilities_1 = require("../../test-utilities");
const Link_1 = require("../Link");
const TextBlock_1 = require("../TextBlock");
const Heading_1 = require("../Heading");
const VisuallyHidden_1 = require("../VisuallyHidden");
const HiddenForAccessibility_1 = require("../HiddenForAccessibility");
const ReviewItem_1 = require("./ReviewItem");
const defaultProps = {
    label: 'Contact',
    children: 'snowdevil@shopify.com',
};
describe('<ReviewItem />', () => {
    describe('label', () => {
        it('renders the label', () => {
            const label = 'Ship to';
            const reviewItem = test_utilities_1.mountWithContext(<ReviewItem_1.ReviewItem {...defaultProps} label={label}/>);
            expect(reviewItem).toContainReactText(label);
        });
    });
    describe('children', () => {
        it('does not render a link when not provided', () => {
            const content = 'snowdevil@shopify.com';
            const reviewItem = test_utilities_1.mountWithContext(<ReviewItem_1.ReviewItem {...defaultProps}>{content}</ReviewItem_1.ReviewItem>);
            expect(reviewItem).toContainReactText(content);
        });
    });
    it('does not render a link when `to` is not provided', () => {
        const reviewItem = test_utilities_1.mountWithContext(<ReviewItem_1.ReviewItem label="Content">snowdevil@shopify.com</ReviewItem_1.ReviewItem>);
        expect(reviewItem).not.toContainReactComponent(Link_1.Link);
    });
    it('renders a link when `to` is provided', () => {
        const to = '/payment';
        const reviewItem = test_utilities_1.mountWithContext(<ReviewItem_1.ReviewItem {...defaultProps} to={to}/>);
        expect(reviewItem).toContainReactComponent(Link_1.Link, {
            to,
        });
    });
    it('renders a hidden label for screen readers when `linkAriaLabel` is provided', () => {
        const reviewItem = test_utilities_1.mountWithContext(<ReviewItem_1.ReviewItem {...defaultProps} to="/information" linkLabel="Change" linkAriaLabel="Change contact information"/>);
        expect(reviewItem.find(Link_1.Link)).toContainReactComponent(VisuallyHidden_1.VisuallyHidden, {
            children: 'Change contact information',
        });
        expect(reviewItem.find(Link_1.Link)).toContainReactComponent(HiddenForAccessibility_1.HiddenForAccessibility, {
            children: 'Change',
        });
    });
});
describe('<ReviewBlock />', () => {
    it('renders the children', () => {
        const reviewBlock = test_utilities_1.mountWithContext(<ReviewItem_1.ReviewBlock>
        <ReviewItem_1.ReviewItem {...defaultProps} label="Ship to"/>
      </ReviewItem_1.ReviewBlock>);
        expect(reviewBlock).toContainReactComponent(ReviewItem_1.ReviewItem);
    });
    it('renders a <Heading /> when a title is passed as a prop', () => {
        const reviewBlock = test_utilities_1.mountWithContext(<ReviewItem_1.ReviewBlock title="Review"/>);
        const heading = reviewBlock.find(Heading_1.Heading);
        expect(heading).toContainReactText('Review');
        expect(reviewBlock).toContainReactComponent('div', {
            'aria-labelledby': heading === null || heading === void 0 ? void 0 : heading.prop('id'),
        });
    });
    it('renders a <VisuallyHidden> when a title and titleHidden are passed as props', () => {
        var _a;
        const reviewBlock = test_utilities_1.mountWithContext(<ReviewItem_1.ReviewBlock title="Review" titleHidden/>);
        const visuallyHidden = reviewBlock.find(VisuallyHidden_1.VisuallyHidden);
        expect(visuallyHidden).toContainReactComponent(TextBlock_1.TextBlock, {
            children: 'Review',
        });
        expect(reviewBlock).toContainReactComponent('div', {
            'aria-labelledby': (_a = visuallyHidden === null || visuallyHidden === void 0 ? void 0 : visuallyHidden.find(TextBlock_1.TextBlock)) === null || _a === void 0 ? void 0 : _a.prop('id'),
        });
    });
});
